@@ -229,7 +229,8 @@ class OwnerAdmin(admin.ModelAdmin):
         "_alliance",
         "character",
         "_webhooks",
-        "has_pings_enabled",
+        "has_default_pings_enabled",
+        "_ping_groups",
         "_is_active",
         "_is_structure_sync_ok",
         "_is_notification_sync_ok",
@@ -237,38 +238,18 @@ class OwnerAdmin(admin.ModelAdmin):
     )
     list_filter = (
         ("corporation__alliance", admin.RelatedOnlyFieldListFilter),
-        "has_pings_enabled",
+        "has_default_pings_enabled",
         "is_active",
         OwnerSyncStatusFilter,
     )
     ordering = ["corporation__corporation_name"]
-    fieldsets = (
-        (
-            None,
-            {
-                "fields": (
-                    "corporation",
-                    "character",
-                    "webhooks",
-                    "is_alliance_main",
-                    "has_pings_enabled",
-                    "is_included_in_service_status",
-                    "is_active",
-                )
-            },
-        ),
-        (
-            "Sync Status",
-            {
-                "classes": ("collapse",),
-                "fields": (
-                    ("structures_last_sync", "structures_last_error",),
-                    ("notifications_last_sync", "notifications_last_error",),
-                    ("forwarding_last_sync", "forwarding_last_error",),
-                ),
-            },
-        ),
-    )
+
+    def _ping_groups(self, obj):
+        names = [x.name for x in obj.ping_groups.all().order_by("name")]
+        if names:
+            return names
+        else:
+            return None
 
     def _corporation(self, obj):
         return obj.corporation.corporation_name
@@ -284,14 +265,12 @@ class OwnerAdmin(admin.ModelAdmin):
     _alliance.admin_order_field = "corporation__alliance__alliance_name"
 
     def _webhooks(self, obj):
-        webhook_names = [x.name for x in obj.webhooks.all().order_by("name")]
-        if webhook_names:
-            return ", ".join(webhook_names)
+        names = [x.name for x in obj.webhooks.all().order_by("name")]
+        if names:
+            return names
         else:
             return None
 
-    _webhooks.short_description = "Webhooks"
-
     def _is_active(self, obj):
         return obj.is_active
 
@@ -337,6 +316,36 @@ class OwnerAdmin(admin.ModelAdmin):
             )
         return self.readonly_fields
 
+    filter_horizontal = ("ping_groups",)
+    fieldsets = (
+        (
+            None,
+            {
+                "fields": (
+                    "corporation",
+                    "character",
+                    "webhooks",
+                    "is_alliance_main",
+                    "has_default_pings_enabled",
+                    "ping_groups",
+                    "is_included_in_service_status",
+                    "is_active",
+                )
+            },
+        ),
+        (
+            "Sync Status",
+            {
+                "classes": ("collapse",),
+                "fields": (
+                    ("structures_last_sync", "structures_last_error",),
+                    ("notifications_last_sync", "notifications_last_error",),
+                    ("forwarding_last_sync", "forwarding_last_error",),
+                ),
+            },
+        ),
+    )
+
     actions = ("update_structures", "fetch_notifications", "send_notifications")
 
     def update_structures(self, request, queryset):
@@ -599,11 +608,7 @@ class StructureAdmin(admin.ModelAdmin):
                 "classes": ("collapse",),
                 "fields": (
                     ("reinforce_hour",),
-                    (
-                        "next_reinforce_hour",
-                        "next_reinforce_weekday",
-                        "next_reinforce_apply",
-                    ),
+                    ("next_reinforce_hour", "next_reinforce_apply",),
                 ),
             },
         ),
@@ -634,12 +639,21 @@ class WebhookAdmin(admin.ModelAdmin):
     list_display = (
         "name",
         "webhook_type",
-        "has_pings_enabled",
+        "has_default_pings_enabled",
+        "_ping_groups",
         "is_active",
         "is_default",
     )
-    list_filter = ("webhook_type", "has_pings_enabled", "is_active")
+    list_filter = ("webhook_type", "has_default_pings_enabled", "is_active")
     save_as = True
+
+    def _ping_groups(self, obj):
+        names = [x.name for x in obj.ping_groups.all().order_by("name")]
+        if names:
+            return names
+        else:
+            return None
+
     actions = ("test_notification", "activate", "deactivate")
 
     def test_notification(self, request, queryset):
@@ -672,3 +686,5 @@ class WebhookAdmin(admin.ModelAdmin):
             self.message_user(request, 'You have de-activated profile "{}"'.format(obj))
 
     deactivate.short_description = "Deactivate selected profiles"
+
+    filter_horizontal = ("ping_groups",)
